from cpgen.graph import *

## Test from basic.py ###
for i in array(10, -10, 100):
    print(i, end=" ")
print("")

for i in arrayreal(10, -100, 100):
    print(i, end=" ")
print("")

for i in matrix(3, 0, 1):
    for j in i:
        print(j, end=" ")
    print("")

for i in zdmatrix(3, -10, 100):
    for j in i:
        print(j, end=" ")
    print("")

print(string(24, 'a', 'z'))

for i in permutation(10):
    print(i, end=" ")

## graph.py ###

for i in tree(10):
    for j in i:
        print(j, end=" ")
    print("")

for i in wtree(10, 1, 100):
    for j in i:
        print(j, end=" ")
    print("")

for i in graph(10, 20):
    for j in i:
        print(j, end=" ")
    print("")




from random import *


# Generate a random array of **INTEGERS** with elements in the range [A, B]
def array(n, a, b):
    arr = [randrange(a, b + 1) for _ in range(n)]
    return arr


# Generate random array of **REAL** with elements in the range [a, b]
def arrayreal(n, a, b):
    arr = [uniform(a, b + 1) for _ in range(n)]
    return arr


# Generate random matrix of size N*N with random integers
'''
# print matrix with space b/w numbers in range [A,B]
for row in matrix:
    print(' '.join(map(str, row)))
'''


# random elements
def matrix(n, a, b):
    m = [[0] * n for _ in range(n)]
    for r in range(n):
        for c in range(n):
            m[r][c] = randint(a, b)
    return m


# with zero at diagonal element
def zdmatrix(n, a, b):
    m = [[0] * n for _ in range(n)]
    for r in range(n):
        for c in range(n):
            m[r][c] = randint(a, b) if r != c else 0
    return m


# symmetric matrix
def symmatrix(n, a, b):
    m = [[0] * n for _ in range(n)]
    for r in range(n):
        for c in range(r + 1):
            m[r][c] = m[c][r] = randint(a, b)
    return m


# Generate a random string from characters in the range [a, b]
def string(n, a, b):
    l = array(n, ord(a), ord(b))
    s = ''
    for char in l:
        s += chr(char)
    return s


# Generate a random permutation of [1, 2 ... N]
def permutation(n):
    p = list(range(1, n + 1))
    shuffle(p)
    return p



from random import *
from cpgen.basic import *


# Generate a random unweighted tree
def tree(n):
    edges = []
    for u in range(2, n + 1):
        v = randrange(1, u)
        edges.append([u, v])

    permute = permutation(n)

    for i in range(0, n - 1):
        u, v = edges[i]
        u = permute[u - 1]
        v = permute[v - 1]
        edges[i] = (u, v)
    return edges


# Generate a random weighted tree
def wtree(n, l, r):
    weigths = array(n - 1, l, r)
    t = tree(n)
    wtree = []

    for i in range(0, n - 1):
        u, v, w = t[i][0], t[i][1], weigths[i]
        wtree.append((u, v, w))

    return wtree


# Undirected, no multiedges and no self-loops
def graph(n, e):
    edges = {}

    if n == 1:
        return []

    for _ in range(e):
        u = randrange(1, n + 1)
        v = u
        while v == u:
            v = randrange(1, n + 1)

        while (u, v) in edges or (v, u) in edges:
            u = randrange(1, n + 1)
            v = u
            while v == u:
                v = randrange(1, n + 1)

        edges[(u, v)] = 1

    ret = []
    for edge in edges:
        ret.append(edge)

    return ret


# Undirected, no multiedges, no self-loops, connected
def connectedgraph(n, e):
    e -= n - 1
    t = tree(n)
    edges = {}
    for edge in t:
        edges[edge] = 1

    for _ in range(e):
        u = randrange(1, n + 1)
        v = u
        while v == u:
            v = randrange(1, e + 1)

        while (u, v) in edges or (v, u) in edges:
            u = randrange(1, n + 1)
            v = u
            while v == u:
                v = randrange(1, n + 1)

        edges[(u, v)] = 1

    ret = []
    for edge in edges:
        ret.append(edge)

    return ret;


# Undirected, no multiedges, no self-loops, can be forced to be connected
def wgraph(n, e, l, r, connected=False):
    g = []
    if not connected:
        g = graph(n, e)
    else:
        g = connectedgraph(n, e)

    weights = array(e, l, r)

    wgraph = []
    for i in range(e):
        u, v, w = g[i][0], g[i][1], weights[i]
        wgraph.append((u, v, w))
    return wgraph
